#!/usr/bin/perl
#
# IP Conntrack Status v0.3 by red_neon
#
# tcp / udp / icmp / igmp / other
#
# INSTALL
#
#	[1] conntrack script
#	# mkdir /etc/srv_scripts
#	# cp conntrack.pl /etc/srv_scripts/
#	# chmod +x /etc/srv_scripts/conntrack.pl
#
#	[2] iptables and rc.local
#	Make sure that iptables is up, check /proc/net/nf_conntrack (or /proc/net/ip_conntrack)
#	# chmod +r /proc/net/nf_conntrack
#	or
#	# chmod +r /proc/net/ip_conntrack
#
#	Add one of available file (nf_conntrack or ip_conntrack) into rc.local, because we will run conntrack.pl not from root (user nobody):
#
#	# nano /etc/rc.local
#	/bin/chmod +r /proc/net/nf_conntrack
#	or
#	/bin/chmod +r /proc/net/ip_conntrack
#
#	[3] first run
#	checking:
#	# su nobody -c "/usr/bin/perl /etc/srv_scripts/conntrack.pl"
#	if is all ok - you will see stats.
#	# cat /tmp/conntrack.stat
#	
#	[4] snmpd
#	# nano /etc/snmp/snmpd.conf
#	add this line:
#	exec conntrack /bin/cat /tmp/conntrack.stat
#
#	restart snmpd:
#	# /etc/init.d/snmpd restart
#
#	[5] cron
#	# nano /etc/crontab
#	add this line:
#	*/5 * * * * nobody /usr/bin/perl /etc/srv_scripts/conntrack.pl > /dev/null 2>&1
#
#	[6] cacti script
#	copy snmp_execscripts.php into cacti/scripts dir
#	for an example my path is /var/share/cacti/scripts
#	# cp snmp_execscripts.php /var/share/cacti/scripts/
#
#	[7] cacti templates
#	import templates into cacti:
#	
#	cacti_graph_template_rn_-_conntrack_status_-_proto_snmp_.xml
#	cacti_graph_template_rn_-_conntrack_status_-_tcp_snmp_.xml
#	cacti_graph_template_rn_-_conntrack_status_-_udp_snmp_.xml
#	cacti_graph_template_rn_-_conntrack_status_-_icmp_snmp_.xml
#

use strict;

my $_tempfile = '/tmp/conntrack.stat';

#################################################
# [ TCP ]
my $_tcp_ss	= 0; # SYN_SENT
my $_tcp_sr	= 0; # SYN_RECV
my $_tcp_e	= 0; # ESTABLISHED
my $_tcp_fw	= 0; # FIN_WAIT
my $_tcp_cw 	= 0; # CLOSE_WAIT
my $_tcp_la 	= 0; # LAST_ACK
my $_tcp_tw	= 0; # TIME_WAIT
my $_tcp_c 	= 0; # CLOSE
#
my $_tcp_u 	= 0; # UNREPLIED
my $_tcp_ha	= 0; # HALF-ASSURED (after UNREPLIED but not ASSURED)
my $_tcp_a 	= 0; # ASSURED
my $_tcp_tot 	= 0; # total tcp
#
# [ UDP ]
my $_udp_u 	= 0;
my $_udp_ha	= 0;
my $_udp_a 	= 0;
my $_udp_tot 	= 0;
#
# [ ICMP ]
my $_icmp_u 	= 0;
my $_icmp_ha	= 0;
my $_icmp_tot 	= 0;
#
# [ IGMP ]
my $_igmp_u 	= 0;
my $_igmp_ha	= 0;
my $_igmp_tot 	= 0;
#
# [ Other ]
my $_other_u 	= 0;
my $_other_ha	= 0;
my $_other_a 	= 0;
my $_other_tot 	= 0;
#
# [ Total ]
my $_tot_u 	= 0;
my $_tot_ha	= 0;
my $_tot_a 	= 0;
my $_tot 	= 0;
####################
my $_output;
my $_ftype;

if (open($_output,'<','/proc/net/nf_conntrack')) {$_ftype=1}
elsif (open($_output,'<','/proc/net/ip_conntrack')) {$_ftype=2}
else{die "Can't open conntrack file, error: $!"}

while (<$_output>) {
	#print ;  #print all connections
	if (/tcp/) {
		$_tcp_e++ 	if /ESTABLISHED/;
		$_tcp_tw++ 	if /TIME_WAIT/;
		$_tcp_fw++ 	if /FIN_WAIT/; 
		$_tcp_ss++ 	if /SYN_SENT/;   
		$_tcp_sr++ 	if /SYN_RECV/; 
		$_tcp_cw++ 	if /CLOSE_WAIT/;
		$_tcp_c++ 	if /CLOSE/;
		$_tcp_la++ 	if /LAST_ACK/;
		if (/ASSURED/) {$_tcp_a++}
		elsif (/UNREPLIED/) {$_tcp_u++}
		else {$_tcp_ha++}
		$_tcp_tot++;
	}elsif (/udp/) {
		if (/ASSURED/) {$_udp_a++}
		elsif (/UNREPLIED/) {$_udp_u++}
		else {$_udp_ha++}
		$_udp_tot++;
	}elsif (/icmp/) {
		# ASSURED is not possible here.. but.. =)
		if (/ASSURED/) {$_other_a++;$_other_tot++} 
		elsif (/UNREPLIED/) {$_icmp_u++;$_icmp_tot++}
		else {$_icmp_ha++;$_icmp_tot++}
	}else{
		my $_igmp=0;
		if  ($_ftype == 1) {
			if  (/^\S+\s+\S+\s+\S+\s+2\s+.+$/) {$_igmp=1}
		}else{
			if  (/^\S+\s+2\s+.+$/) {$_igmp=1}
		}
		if  ($_igmp == 1) {
			# ASSURED is not possible here.. but.. =)
			if (/ASSURED/) {$_other_a++;$_other_tot++} 
			elsif (/UNREPLIED/) {$_igmp_u++;$_igmp_tot++}
			else {$_igmp_ha++;$_igmp_tot++}
		}else{
			if (/ASSURED/) {$_other_a++}
			elsif (/UNREPLIED/) {$_other_u++}
			else {$_other_ha++}
			$_other_tot++;
		}
	}
	if (/ASSURED/) {$_tot_a++}
	elsif (/UNREPLIED/) {$_tot_u++}
	else {$_tot_ha++}	
	$_tot++;
}

close $_output or warn "Error closing conntrack file: $!";
####################
print "[TCP]:
established:$_tcp_e
timewait:$_tcp_tw
finwait:$_tcp_fw
synsent:$_tcp_ss
synrecv:$_tcp_sr
closewait:$_tcp_cw
close:$_tcp_c
lastack:$_tcp_la
#
unreplied:$_tcp_u
half-assured:$_tcp_ha
assured:$_tcp_a
total:$_tcp_tot
##
[UDP]:
unreplied:$_udp_u
half-assured:$_udp_ha
assured:$_udp_a
total:$_udp_tot
##
[ICMP]:
unreplied:$_icmp_u
half-assured:$_icmp_ha
total:$_icmp_tot
##
[IGMP]:
unreplied:$_igmp_u
half-assured:$_igmp_ha
total:$_igmp_tot
##
[Other]:
unreplied:$_other_u
half-assured:$_other_ha
assured:$_other_a
total:$_other_tot
##
[IP total]:
unreplied:$_tot_u
half-assured:$_tot_ha
assured:$_tot_a
total:$_tot
";
####################
my $_tcp_gr="tcp_ss:$_tcp_ss tcp_sr:$_tcp_sr tcp_e:$_tcp_e tcp_fw:$_tcp_fw tcp_cw:$_tcp_cw tcp_la:$_tcp_la tcp_tw:$_tcp_tw tcp_c:$_tcp_c tcp_a:$_tcp_a tcp_u:$_tcp_u tcp_ha:$_tcp_ha tcp_tot:$_tcp_tot";
my $_udp_gr="udp_a:$_udp_a udp_u:$_udp_u udp_ha:$_udp_ha udp_tot:$_udp_tot";
my $_icmp_gr="icmp_u:$_icmp_u icmp_ha:$_icmp_ha icmp_tot:$_icmp_tot";
my $_igmp_gr="igmp_u:$_igmp_u igmp_ha:$_igmp_ha igmp_tot:$_igmp_tot";
my $_other_gr="other_a:$_other_a other_u:$_other_u other_ha:$_other_ha other_tot:$_other_tot";
my $_tot_gr="tot_a:$_tot_a tot_u:$_tot_u tot_ha:$_tot_ha tot:$_tot";
my $_save;
open($_save,'>',$_tempfile) or die "Can't write data to file: \"$_tempfile\", error: $!";
print $_save "$_tcp_gr $_udp_gr $_icmp_gr $_igmp_gr $_other_gr $_tot_gr\n";
close $_save or warn "Error closing file: \"$_tempfile\", error: $!";
####################
my $_ls=`which ls`;
if ($_ls =~ /^$/){die "fail with cmd: which ls"}
else{$_ls =~ s/\n$//}
my $_checkright = `$_ls -l $_tempfile`;
if ($_checkright !~ /^.rw.rw.rw..+/) {
	`chmod 777 $_tempfile`;
	print "ok! cmd: chmod 777 $_tempfile\n";
}
####################
